/*
 * libstruct.h - internal header file.
 * List of functions that are exported from
 * the application to libiconv
 *
 * Copyright (C) 2018 Thorsten Otto
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBICONVSTRUCT_H__
#define __LIBICONVSTRUCT_H__ 1

#ifndef _CDECL
#define _CDECL
#endif

#include <mint/slb.h>


struct _iconvslb_funcs {
	/*
	 * sizeof(this struct), as
	 * used by the application.
	 */
	size_t struct_size;
	/*
	 * sizeof of an int of the caller.
	 * As for now, must match the one which
	 * was used to compile the library (32 bit)
	 */
	size_t int_size;
	/*
	 * version of iconv.h the caller used.
	 * As for now, should match the version that was
	 * used to compile the library.
	 */
	long iconv_vernum;
	
	iconv_int_t _CDECL (*p_get_errno)(void);
	void _CDECL (*p_set_errno)(iconv_int_t e);

	void *_CDECL (*p_memset)(void *, iconv_int_t, size_t);
	void *_CDECL (*p_memcpy)(void *, const void *, size_t);
	iconv_int_t _CDECL (*p_memcmp)(const void *, const void *, size_t);
	
	size_t _CDECL (*p_strlen)(const char *);
	char *_CDECL (*p_strcpy)(char *, const char *);
	iconv_int_t _CDECL (*p_strcmp)(const char *, const char *);
	char *_CDECL (*p_strchr)(const char *, iconv_int_t);

	void *_CDECL (*p_malloc)(size_t);
	void _CDECL (*p_free)(void *);

	char *_CDECL (*p_getenv)(const char *);
	void _CDECL (*p_qsort)(void *, size_t, size_t, iconv_int_t _CDECL (*compar)(const void *, const void *));

	size_t _CDECL (*p_wcrtomb)(char *s, iconv_wchar_t wc, mbstate_t *ps);
	size_t _CDECL (*p_mbrtowc)(wchar_t *pwc, const char *s, size_t n, mbstate_t *ps);
	iconv_int_t _CDECL (*p_mbsinit)(const mbstate_t *ps);
	
	/* room for later extensions */
	void *unused[45];
};

extern struct _iconvslb_funcs iconvslb_funcs;

long _CDECL libiconv_slb_control(long fn, void *arg);

#endif
